<?php

/****************************************************************************
	CLASS:	OBJECT
	PURPOSE:	master function for managing docmgr objects.  this includes
				    creating, saving, update, moving, or deleting
****************************************************************************/

class DOCMGR_OBJECT extends DOCMGR_AOBJECT
{

	public function get()
	{
	
		$class = "DOCMGR_".$this->objectInfo["object_type"];
		
		//just init the appropriate class based on our type and call 
		//its get method
		$o = new $class($this->apidata);
		$o->get();
	
	}	

	public function save()
	{

		//guest accounts stop here
		if (PERM::check(GUEST_ACCOUNT,1))
		{
			$this->throwError("You do not have permissions to manage this object");
			return false;
		}

		//create or update
		if ($this->objectId) 
		{

			//make sure we have permissions to manage this object
			if (!$this->permCheck("admin"))
			{
				$this->throwError("You do not have permissions to manage this object");
				return false;
			}
		
			$this->update($this->apidata);

			$event = OBJ_UPDATED;
			
		}
		else 
		{

			$this->create($this->apidata);
			$event = OBJ_CREATED;
			
		}
		
		//stop here if no object id
		if (!$this->objectId) return false;

		//store the content
		$objType = $this->objectInfo["object_type"];
		$objClass = "DOCMGR_".$objType;

		//init our subclass and call it's update routine		
		$o = new $objClass($this->objectId);
		$o->update($this->apidata);

		//check for errors from update
		$err = $o->getError();

		if ($err)
		{

			//problem.  delete the new object and bail
			$this->delete();			
			$this->throwError($err[0],$err[1]);		

			return false;
					
		}

		//save keywords
		$k = new DOCMGR_UTIL_KEYWORD($this->objectId);
		$k->saveValues($this->apidata);

		//queue the object for indexing and thumbnailing
		$i = new DOCMGR_OBJINDEX($this->objectId);
		$i->run(USER_ID);

		$objClass = "DOCMGR_".$objType;

		//don't thumbnail in the background
		if (defined("DISABLE_BACKTHUMB")) $o->thumb();
			
		//lock if necessary
		if ($this->apidata["unlock"]) 
		{
			$l = new DOCMGR_UTIL_LOCK($this->objectId);
			$l->clear();
		}
                                      
		//log the event
		if ($event) logEvent($event,$this->objectId);

		//make sure we output the object id and path
		$this->PROTO->add("object_id",$this->objectId);
		$this->PROTO->add("object_path",$this->path);	

		$this->updateCollectionSizes();

		//return the object id for internally called functions
		return $this->objectId;

	}


	/****************************************************************************
		FUNCTION:	create
		PURPOSE:	creates a new object in the system
		INPUTS:		object_type -> collection,file,document,etc
				      name	-> object name
				      parent_id -> id of parent we store in (id or array)
				      summary -> object summary
				      object_owner -> account that will own this object
	****************************************************************************/

	protected function create($data) 
	{

		//extract all our data into easy to use variables arr["keyname"] => $keyname
		extract($data);

		//make sure they have permissions to insert objects
		if (!PERM::check(INSERT_OBJECTS))
		{
			$msg = "You do not have permissions to create objects";
			$this->throwError($msg);
			return false;
		}

		if (!$data["parent_id"] && $data["parent_path"]) 
		{

			$info = $this->objectFromPath($data["parent_path"]);
			$parent_id = $info["id"];

			//if no parent id and we are asked to make one on the spot, handle it
			if ($parent_id == null && $data["mkdir"]) 
			{
				$parent_id = DOCMGR_UTIL_COMMON::recurMkParent($data["parent_path"]);
			} 
				
		}

		//if moving to the root collection, make sure they have permission to do that
		if ($parent_id=="0" && !PERM::check(CREATE_ROOT))
		{
			$msg = "You do not have permissions to create objects in the root folder";
			$this->throwError($msg);
			return false;
		}
				

		if (!$name) {
			$this->throwError("Object name not specified");
			return false;
		}

		//no parent
		if ($parent_id==NULL) 
		{
			$this->throwError("No valid collection was passed to create object in");
			return false;
		}    

		//if passed exist override, come up with our new filename if one already exists in this collection
		if ($this->apidata["exist_rename"]) {

			//keep going until we find one that doesn't exist 
			while (!checkObjName($this->conn,$name,$parent_id)) $name = $this->incrementName($name); 
 
		//make sure there isn't already an object with this name
		} else if (!checkObjName($this->conn,$name,$parent_id)) {

			//get the object's info and bail returning the existing object's id
			$sql = "SELECT id FROM docmgr.dm_view_objects WHERE parent_id='$parent_id' AND name='".$name."'";
			$info = $this->DB->single($sql);

			$this->throwError("Object with this name \"".$name."\" already exists in this collection");
			$this->setObjectId($info["id"]);

			return false;
			
		}

		//make sure we have permissions to create in this collection
		if ($parent_id==0) 
		{
			if (defined("ADMIN_ROOTLEVEL") && !PERM::check(ADMIN))
			{
		 		$this->throwError("You do not have permissions to create objects in this collection");
		 		return false;
			}
		} 
		else 
		{
		 	$cb = DOCMGR_UTIL_OBJPERM::getUser($parent_id);
		 	if (!DOCMGR_UTIL_OBJPERM::check($cb,"edit"))
		 	{
				$this->throwError("You do not have permissions to create objects in this collection");
				return false;
			}
		}
			  
		//some default values
		if (!$object_owner) $object_owner = USER_ID;
	 	if (!$protected) $protected = "f";
		if (!$hidden) $hidden = "f";
			 	 
		$this->DB->begin();

		//insert into the main object table
		$option = null;
		$option["name"] = $name;
		$option["summary"] = $summary;
		$option["version"] = "0";
		$option["create_date"] = date("Y-m-d H:i:s");
		$option["object_type"] = $object_type;
		$option["object_owner"] = $object_owner;
		$option["last_modified"] = date("Y-m-d H:i:s");
		$option["modified_by"] = USER_ID;
		$option["protected"] = $protected;
		$option["hidden"] = $hidden;

		//insert the collection
		$this->objectId = $this->DB->insert("docmgr.dm_object",$option,"id");

		//store objectId in parent for other classes to use
		$this->setObjectId($this->objectId);
		
		//figure out the directory levels for this object.  We do this here for all objects 
		//until I think of a way that doesn't involve sequences
		if ($object_type!="collection") 
		{

			$level = $this->storeLevel();
			$data["object_directory"] = $level;

		}

		//setup the parent link for the collection
		$sql = "INSERT INTO docmgr.dm_object_parent (object_id,parent_id) VALUES ('".$this->objectId."','$parent_id');";
		$this->DB->query($sql);

		//inherit the parent's permissions
		if ($this->apidata["noinherit"]) DOCMGR_UTIL_OBJPERM::setDefault($this->objectId,$this->apidata["private"]);
		else DOCMGR_UTIL_OBJPERM::inherit($this->objectId,$parent_id,$this->apidata["private"]);

		$this->DB->end();

		//send out an alert.  We send the alert out for the object, even though the
		//parent triggers it.  This allows a user to see the new file right away
		if ($object_type=="collection" || $object_type=="url" || $object_type=="document") 
			sendEventNotify($this->objectId,OBJ_CREATE_ALERT,$parent_id);

		$this->setObjectInfo();

		//look for db errors
		$err = $this->DB->error();

		if ($err) 
		{
			$this->throwError($err);
			return false;
		} 
		else 
		{
			return $this->objectId;
		}
			
	}

	/****************************************************************************
		FUNCTION:	update
		PURPOSE:	updates a new object in the system
		INPUTS:		object_type -> collection,file,document,etc
				      name	-> object name
				      parent_id -> id of parent we store in (id or array)
				      summary -> object summary
				      object_owner -> account that will own this object
	****************************************************************************/
	protected function update($data) 
	{

		//extract all our data into easy to use variables arr["keyname"] => $keyname
		extract($data);

		//sanity checking
		if (!$object_type) 
		{

			//try to pull object type from our objectinfo if not set
			if ($this->objectInfo) $object_type = $this->objectInfo["object_type"];
			else 
			{
				$this->throwError("Object type not specified");
				return false;
			}
			
		}

		if (!$this->objectId) 
		{
			$this->throwError("Object id not specified");
			return false;
		}

		//make sure there isn't already an object with this name (if we are renaming)
		if ($name && !checkObjName($this->conn,$name,$parent_id,$this->objectId)) 
		{
			$this->throwError("Object with this name already exists in this collection");
			return false;
		}

		//if it's "No summary available", blank it
		if ($summary==_NO_SUMMARY_AVAIL) $summary = null;

		//add in the object path
		$data["object_directory"] = $this->getObjectDir();

		$this->DB->begin();

		//insert into the main object table
		$option = null;
		if ($name) $option["name"] = $name;
		if ($summary) $option["summary"] = $summary;
		$option["last_modified"] = date("Y-m-d H:i:s");
		$option["modified_by"] = USER_ID;
		$option["where"] = "id='".$this->objectId."'";
		$this->DB->update("docmgr.dm_object",$option);

		logEvent(OBJ_PROP_UPDATE,$this->objectId);

		$this->DB->end();


		//look for db errors
		$err = $this->DB->error();
		if ($err) 
		{
			$this->throwError($err);
			return false;
		} 
		else 
		{
			return $this->objectId;
		}

	}


	function index()
	{

		if ($this->apidata["prop_only"]) $p = true;
		else $p = false;

		$o = new DOCMGR_OBJINDEX($this->objectId);
		$o->index($p);
	
	}

	function thumb()
	{

		$objClass = "DOCMGR_".$this->objectInfo["object_type"];
		
		$OBJ = new $objClass($this->objectId);
		$OBJ->thumb();
	
	}
	
	/***************************************************************************
		deleting functions
	***************************************************************************/

	/****************************************************************************
		FUNCTION:	delete
		PURPOSE:	deletes an object in the system
	****************************************************************************/
	public function delete()
	{

		//convert to array if not passed as one
	  if (is_array($this->objectId)) $objarr = $this->objectId;
	  else $objarr = array($this->objectId);

	  $oldParents = array();

		//loop through our objects and delete
		foreach ($objarr AS $obj) 
		{

			//init for testing later		
			$LOCK = new DOCMGR_UTIL_LOCK($obj);

		  //figure out what kind of object this is
		  $sql = "SELECT id,name,object_type,protected FROM docmgr.dm_object WHERE id='".$obj."'";
		  $info = $this->DB->single($sql);
	
		  //if we can't find the obj, there's been an error.  Get out!!!	
		  if (!$info) {
	
		    $this->throwError("Could not find object \"".$info["name"]."\" for removal");
		    return false;
	
		  } else if ($LOCK->isLocked($info["id"])) {
	
		  	$this->throwError("\"".$info["name"]."\" is currently locked","OBJ_LOCK");
		  	return false;

		  } else if ($info["protected"]=="t") {
	
		  	$this->throwError("\"".$info["name"]."\" is a protected object and cannot be deleted");
		  	return false;
	
		  }

		  //if told to bypass perm check, do it.  Note, this can only be called internally,
		  //and is currently only used on object storage folder creation
		  if (!$noperms)
		  {

				//make sure we are allow to edit anything in the destination folder, since we are 
			  //moving things there
			  $cb = DOCMGR_UTIL_OBJPERM::getUser($obj);
			  if (!DOCMGR_UTIL_OBJPERM::check($cb,"admin"))
			  {
			    $this->throwError("You do not have permissions to delete this ".$info["name"]);
					return false;
				}

			}
			
			$this->DB->begin();
	
		  $className = "DOCMGR_".$info["object_type"];
	
		  //now hand off to our other functions for the object-specific processing
		  $c = new $className($obj);
		  $c->remove();

		  //get the old parents of this object
		  $sql = "SELECT parent_id FROM docmgr.dm_object_parent WHERE object_id='$obj'";
		  $info = $this->DB->fetch($sql,1);
		  $oldParents = array_merge($oldParents,$info["parent_id"]);


			//all the tables we need to clear references too	
		  $arr = array(	"docmgr.dm_index_queue","docmgr.dm_object_perm","docmgr.dm_object_parent",
		  							"docmgr.dm_index","docmgr.keyword_value","docmgr.dm_alert",
										"docmgr.dm_bookmark","docmgr.dm_discussion","docmgr.dm_subscribe",
										"docmgr.dm_properties","docmgr.dm_locks","docmgr.dm_locktoken");
		 
		  $sql = null;               
		                
		  foreach ($arr AS $table) 
		  {
		    $sql .= "DELETE FROM $table WHERE object_id='".$obj."';";
		  }
		  
		  //related files
		  $sql .= "DELETE FROM docmgr.dm_object_related WHERE object_id='".$obj."' OR related_id='".$obj."';";
	
		  //primary object removal and associated table entry removal
		  $sql .= "DELETE FROM docmgr.dm_object WHERE id='".$obj."'; ";

		  //run the query
		  $this->DB->query($sql);

		  $this->DB->end();
	
		  //look for temp folders and delete them
		  $tempname = ".object".$obj."_storage";

		  $sql = "SELECT id FROM docmgr.dm_object WHERE name='$tempname' AND object_type='collection' AND hidden='t'";
		  $info = $this->DB->fetch($sql);
			for ($i=0;$i<$info["count"];$i++) $this->delete($info[$i]["id"]);	
	
			//look for errors
		  $err = $this->DB->error();
		  if ($err) 
		  {
		    $this->throwError($err);
				break;
			} 
			else 
			{

		    //send out an alert
		    sendEventNotify($obj,OBJ_REMOVE_ALERT);
	
		  }
		  
		}	//end for loop

		//update everyone's size
		$this->updateCollectionSizes($oldParents);

	}	


	/****************************************************************************
		FUNCTION:	move
		PURPOSE:	relocates an object to a new parent or parents
	****************************************************************************/
	public function move() 
	{

		//convert our object id into an array.  this is an array of objects we are moving
		if (is_array($this->objectId)) $objectArr = $this->objectId;
		else $objectArr = array($this->objectId);

		//now get the parent id from the parent_path parameter
		if ($this->apidata["parent_id"]) $parent = $this->apidata["parent_id"];
		else 
		{

			$info = $this->objectFromPath($this->apidata["parent_path"]);

			if (!$info) 
			{

				$msg = "Error moving object.  Could not find parent in system";
				$this->throwError($msg);
				return false;

			} 
			else $parent = $info["id"];

		}

		//if moving to the root collection, make sure they have permission to do that
		if ($parent=="0" && !PERM::check(CREATE_ROOT))
		{
			$msg = "You do not have permissions to move objects to the root folder";
			$this->throwError($msg);
			return false;
		}
	
		//make sure we are not moving something to a category where an object with its name already exists
		//there may be a more efficient way to do this without so many queries to the db
		foreach ($objectArr AS $curObj) 
		{

			$sql = "SELECT name FROM docmgr.dm_object WHERE id='$curObj'";
			$info = $this->DB->single($sql);

			if (!checkObjName($this->conn,sanitize($info["name"]),$parent)) 
			{
				$this->throwError($info["name"]." already exists in destination collection");
				return false;
			}
	
		}
	
	
		//make sure we are allow to edit anything in the destination folder, since we are 
		//moving things there
		$cb = DOCMGR_UTIL_OBJPERM::getuser($parent);
		if (!$this->permCheck("edit",$cb))
		{
			$this->throwError("You do not have permissions to move this object to the destination folder");
			return false;
		}

		//start the move
		$this->DB->begin();

		//get the old parents for later
		$sql = "SELECT parent_id FROM docmgr.dm_object_parent WHERE object_id IN (".implode(",",$objectArr).");";
		$info = $this->DB->fetch($sql,1);
		$oldParents = $info["parent_id"];

		//delete all current objects
		$sql = "DELETE FROM docmgr.dm_object_parent WHERE object_id IN (".implode(",",$objectArr).");";
		$this->DB->query($sql);

		$sql = null;

		//insert with our new parent
		for ($row=0;$row<count($objectArr);$row++) 
		{

			//make sure we have permissions to move this object
			$cb = DOCMGR_UTIL_OBJPERM::getUser($objectArr[$row]);
			if (!DOCMGR_UTIL_OBJPERM::check($cb,"admin")) continue;

			$sql .= "INSERT INTO docmgr.dm_object_parent 
		        	(object_id,parent_id) 
		        	VALUES 
		        	('".$objectArr[$row]."','".$parent."');";

		}

		if ($sql) $this->DB->query($sql);

		//update collection sizes
		$this->updateCollectionSizes($objectArr);
		$this->updateCollectionSizes($oldParents);

		//log it
		for ($row=0;$row<count($objectArr);$row++) 
		{
			logEvent(OBJ_MOVED,$objectArr[$row]);
		}
						
		$this->DB->end();

		$err = $this->DB->error();
		if ($err) $this->throwError($err);
	
	}
	
	/****************************************************************************
		FUNCTION:	getInfo
		PURPOSE:	adds generic data object into the output queue
	****************************************************************************/
	public function getInfo() 
	{

		if (!$this->objectId) return false;

		//make some pretty fields for viewing
		$a = new ACCOUNT();
		$arr = $a->getInfo($this->objectInfo["modified_by"]);

		$this->objectInfo["view_modified_by"] = $arr["first_name"]." ".$arr["last_name"];
		$this->objectInfo["view_last_modified"] = dateView($this->objectInfo["last_modified"]);

		$this->objectInfo["object_path"] = $this->path;
		
		$arr = $a->getInfo($this->objectInfo["object_owner"]);
		$this->objectInfo["owner_name"] = $arr["first_name"]." ".$arr["last_name"];

		//get the parents of this object
		$sql = "SELECT parent_id FROM docmgr.dm_object_parent WHERE object_id='".$this->objectId."'";
		$info = $this->DB->fetch($sql);

		//pass as an array
		for ($i=0;$i<$info["count"];$i++) 
		{
			$this->objectInfo["parents"][$i] = $info[$i]["parent_id"];
		}

		//merge in view info for collections (list, thumb, whatever)
		if ($this->objectInfo["object_type"]=="collection") 
		{
			$this->objectInfo = @array_merge($this->objectInfo,$this->getView());
		}

		//merge in lock info
		$arr = array();
		$arr["count"] = 1;
		$arr[0] = $this->objectInfo;

		$l = new DOCMGR_UTIL_LOCK();
		$l->addToObject($arr);

		$this->objectInfo = $arr[0];

		//add to our output stream		
		$this->PROTO->add("object",$this->objectInfo);

		//return for internal calls
		return $this->objectInfo;
	
	}
	
	/****************************************************************************
		FUNCTION:	getId
		PURPOSE:	returns an xml string containing the object id
	****************************************************************************/
	public function getId() 
	{
	
		$this->PROTO->add("object_id",$this->objectId);
			
	}
				
	/****************************************************************************
		FUNCTION:	saveParent
		PURPOSE:	stores a new parent for an object
	****************************************************************************/
	public function saveParent($pid=null) 
	{

		//perm check
		if (!$this->permCheck("admin"))
		{
		  $this->throwError("You do not have permissions to manage this object");
			return false;
		}

		//if not passed directly, snag from xml 
		if (!$pid) $pid = $this->apidata["parent_id"];

		//make array
		if (!is_array($pid)) $pid = array($pid);

		//object can't be moved into itself		
		if (in_array($this->objectId,$pid)) 
		{
			$this->throwError("An object cannot be it's own parent");
			return false;
		}

		//we are doing an update, make sure an object with the same name does not already exist
		if (!checkObjName($this->conn,addslashes($this->objectInfo["name"]),$pid,$this->objectId)) 
		{
		  $this->throwError(ERROR_MESSAGE);
		  return false;
		}

		//get the old parents for later
		$sql = "SELECT parent_id FROM docmgr.dm_object_parent WHERE object_id IN (".implode(",",$objectArr).");";
		$info = $this->DB->fetch($sql,1);
		$oldParents = $info["parent_id"];
	
		//remove the old entries
		$sql = "DELETE FROM docmgr.dm_object_parent WHERE object_id='".$this->objectId."';";
	
		//insert new entries
		foreach ($pid AS $p) 
		{

			$cb = DOCMGR_UTIL_OBJPERM::getUser($p);
			if (!PERM::is_set($cb,OBJ_ADMIN))
			{
			    $this->throwError("You do not have permissions to move this object to the destination folder");
				  return false;
		  }

		  $sql .= "INSERT INTO docmgr.dm_object_parent (object_id,parent_id) VALUES ('".$this->objectId."','".$p."');";

		}

		//run the query	
		if (!$this->DB->query($sql)) 
		{
		  $this->throwError("Unble to update object parent");
		  return false;
		}
		else
		{

		
			$this->updateCollectionSizes();
			$this->updateCollectionSizes($oldParents);
		
		}
		
	}



	/**********************************************************
		for converting an object to a new type
	**********************************************************/
	public function convert() 
	{

		//must have valid object
		if (!$this->objectInfo) 
		{
			$this->throwError("Invalid object passed");
			return false;
		}

		//only allow document and file conversion
		if ($this->objectInfo["object_type"]!="file" && $this->objectInfo["object_type"]!="document") 
		{
			$this->throwError("You may only convert files or documents");
			return false;
		}

		//did we pass the type we want to convert to
		if (!$this->apidata["to"]) {
			$this->throwError("You did not specifiy what you want to convert the file to");
			return false;
		}

		if ($this->objectInfo["object_type"]=="document") {				
			$ext = "html";
			$newname = $this->objectInfo["name"].".".$this->apidata["to"];
		}
		else {
			$ext = return_file_extension($this->objectInfo["name"]);
			$newname = str_replace(".".$ext,".".$this->apidata["to"],$this->objectInfo["name"]);
		}
		
		//bail if no extension
		if (!$ext) 
		{
			$this->throwError("Object's extension could not be determined");
			return false;
		}

		//get our file from docmgr and setup a working file to play with		
		$worker = $this->setupWorker();

		//fire up openoffice
		$oo = new OPENOFFICE($worker,$this->objectId);
		$newfile = $oo->convert($this->apidata["to"]);

		//return the content if asked
		if ($this->apidata["return"]=="content") 
		{

			$this->PROTO->add("content",file_get_contents($newfile));

		} 
		else 
		{

			//get our file type to pass to the browser
			if ($type = return_file_mime(strtolower($newname))) header ("Content-Type: $type");
			else $type="application/octet-stream";
	 
			// send headers to browser to initiate file download
			header ("Content-Type: ".$type);
			header ("Content-Type: application/force-download");
			header ("Content-Length: ".filesize($newfile));
			header ("Content-Disposition: attachment; filename=\"$newname\"");
			header ("Content-Transfer-Encoding:binary");
			header ("Cache-Control: must-revalidate, post-check=0, pre-check=0");
			header ("Pragma: public");
			readfile_chunked($newfile);
			die;

		} 
			                                
	}

	protected function setupWorker() 
	{

		//make our temp directory
		$tmpdir = TMP_DIR."/".USER_LOGIN;
		recurmkdir($tmpdir);
	
		//get the path to our document
		if ($this->objectInfo["object_type"]=="document") {

			$realname = $this->objectInfo["name"];
			$version = $this->objectInfo["version"];
 
			$sql = "SELECT id FROM docmgr.dm_document WHERE object_id='".$this->objectId."' AND version='$version'";
			$info = $this->DB->single($sql);
			$documentId = $info["id"];

			// get the filename
			$filename = DOC_DIR."/".$this->getObjectDir()."/".$documentId.".docmgr";

			$dest = $tmpdir."/worker.html";

		//get path to our file		
		} else {

			$realname = $this->objectInfo["name"];
			$version = $this->objectInfo["version"];
	
			$sql = "SELECT id FROM docmgr.dm_file_history WHERE object_id='".$this->objectId."' AND version='$version'";
			$info = single_result($this->conn,$sql);
			$file_id = $info["id"];
	
			// get the filename
			$filename = DATA_DIR."/".$this->getObjectDir()."/".$file_id.".docmgr";
			$ext = return_file_extension($realname);
			
			$dest = $tmpdir."/worker.".$ext;
				              		
		}

		//copy to working location
		copy ($filename,$dest);                               		

		return $dest;
	
	}

	public function getSubscription() {
	
		$sql = "SELECT * FROM docmgr.dm_subscribe WHERE account_id='".USER_ID."' AND object_id='".$this->objectId."';";
		$list = $this->DB->fetch($sql);
		
		for ($i=0;$i<$list["count"];$i++) 
		{
			$this->PROTO->add("subscribe",$list[$i]);
		}

		//look for db errors	
		$err = $this->DB->error();
		if ($err) $this->throwError($err);
		
	}


	public function updateSubscription() {

		$type = $this->apidata["type"];

		$sql = "DELETE FROM docmgr.dm_subscribe WHERE object_id='".$this->objectId."';";
		
		if ($type) {

			if (!is_array($type)) $type = array($type);
			if (!$this->apidata["send_email"]) $this->apidata["send_email"] = "f";
			if (!$this->apidata["send_file"]) $this->apidata["send_file"] = "f";
		
			//pull our settings from our checkbox
			for ($i=0;$i<count($type);$i++) 
			{

				$opt = null;
				$opt["object_id"] = $this->objectId;
				$opt["account_id"] = USER_ID;
				$opt["send_email"] = $this->apidata["send_email"];
				$opt["event_type"] = $type[$i];
				$opt["send_file"] = $this->apidata["send_file"];
				$opt["query"] = 1;
				$sql .= $this->DB->insert("docmgr.dm_subscribe",$opt).";";

			}

		}

		$this->DB->query($sql);		

		//look for db errors
		$err = $this->DB->error();
		if ($err) $this->throwError($err);

	}

	/****************************************************
		FUNCTION: createLink
		PURPOSE:	returns a link non-users can use to view 
							a file or document
	****************************************************/
	public function createLink()
	{

		//make sure it's a valid object type
		if ($this->objectInfo["object_type"]!="document" && $this->objectInfo["object_type"]!="file")
		{
			$err = "You can only create links for documents and files";
		} else {
		
			$time = time();

			//generate a unique hash		
			$link = md5($time.USER_LOGIN);
			$expires = $time + ($this->apidata["expire"] * 3600); 
			
			//first create a valid link reference in the database
			$opt = null;
			$opt["object_id"] = $this->objectId;
			$opt["link"] = $link;
			$opt["account_id"] = USER_ID;
			$opt["created"] = date("Y-m-d H:i:s",$time);
			$opt["expires"] = date("Y-m-d H:i:s",$expires);
			$this->DB->insert("docmgr.object_link",$opt);
			
			$url = SITE_URL."api.php?viewobj=".$link;	

			$err = $this->DB->error();
				
		}

		//return the error if there is one, otherwise show the link		
		if ($err) $this->throwError($err);
		else $this->PROTO->add("object_link",$url);

	}
	
	public function getView()
	{

		//bail if no object
		if (!$this->objectId) return false;

		$ret = array();
	
		$sql = "SELECT * FROM docmgr.object_view WHERE object_id='".$this->objectId."' AND account_id IN ('0','".USER_ID."')";
		$list = $this->DB->fetch($sql,1);

		if ($list["count"]>0)
		{

			//default view			
			$key = array_search("0",$list["account_id"]);
			if ($key!==FALSE) $ret["default_view"] = $list["view"][$key];
			
			//account view
			$key = array_search(USER_ID,$list["account_id"]);
			if ($key!==FALSE) $ret["account_view"] = $list["view"][$key];

		}

		return $ret;
	
	}

	public function getAlerts() 
	{

		$xml = file_get_contents("config/alerts.xml");
		$arr = XML::decode($xml);
		$num = count($arr["alert"]);
		
		$sql = "SELECT dm_alert.*,name,object_type FROM docmgr.dm_alert
						LEFT JOIN docmgr.dm_object ON dm_alert.object_id = dm_object.id
						WHERE dm_alert.account_id='".USER_ID."'";
		$list = $this->DB->fetch($sql);
		
		for ($i=0;$i<$list["count"];$i++) 
		{

			//get the alert description
			for ($c=0;$c<$num;$c++)
			{

				if ($arr["alert"][$c]["link_name"]==$list[$i]["alert_type"])
				{			
					$list[$i]["description"] = $arr["alert"][$c]["name"];
					break;
				}	
			
			}

			$this->PROTO->add("alert",$list[$i]);

		}

		//look for db errors	
		$err = $this->DB->error();
		if ($err) $this->throwError($err);
		
	}

	public function getAllSubscriptions() 
	{

		$sql = "SELECT DISTINCT dm_subscribe.object_id,name,object_type FROM docmgr.dm_subscribe
						LEFT JOIN docmgr.dm_object ON dm_subscribe.object_id = dm_object.id
						WHERE dm_subscribe.account_id='".USER_ID."'";
		$list = $this->DB->fetch($sql);

		for ($i=0;$i<$list["count"];$i++) 
		{

			$this->PROTO->add("subscribe",$list[$i]);

		}

		//look for db errors	
		$err = $this->DB->error();
		if ($err) $this->throwError($err);
		
	}

	public function clearAlert()
	{

		//the bit at the end keeps users from deleting other people's alerts
		$sql = "DELETE FROM docmgr.dm_alert WHERE id='".$this->apidata["alert_id"]."' AND account_id='".USER_ID."'";
		$this->DB->query($sql);	

		//look for db errors	
		$err = $this->DB->error();
		if ($err) $this->throwError($err);
	
	}

	protected function storeLevel()
	{
	
		$level1 = $this->DB->increment_seq("docmgr.level1");
		$level2 = $this->DB->increment_seq("docmgr.level2");

		//this should never change for an object, but we'll pass a delete query just to be safe"
		$sql = "DELETE FROM docmgr.dm_dirlevel WHERE object_id='".$this->objectId."';
						INSERT INTO docmgr.dm_dirlevel (object_id,level1,level2) VALUES ('".$this->objectId."','$level1','$level2');";
		$this->DB->query($sql);                 

		$level = $level1."/".$level2;

	}

	public function getPerms()
	{
	
		$cb = DOCMGR_UTIL_OBJPERM::getUser($this->objectId);

		if (!DOCMGR_UTIL_OBJPERM::check($cb,"admin")) 
		{
			$this->throwError("You do not have permission to manage this object");
		}
		else
		{
			DOCMGR_UTIL_OBJPERM::getList($this->objectId,$this->apidata);
		}
			
	}

	public function savePerms()
	{
	
		//make sure they have permissions to manage the object.  if not, bail          
		$cb = DOCMGR_UTIL_OBJPERM::getUser($this->objectId);

		if (!DOCMGR_UTIL_OBJPERM::check($cb,"admin")) 
		{
			$this->throwError("You do not have permission to manage this object");
		}
		else
		{

			DOCMGR_UTIL_OBJPERM::save($this->objectId,$this->apidata["perm"],$this->apidata["reset_perms"]);

			logEvent(OBJ_PERM_UPDATE,$this->objectId);

		}
			
	}

	public function trash()
	{

		//convert our object id into an array.  this is an array of objects we are moving
		if (is_array($this->objectId)) $objectArr = $this->objectId;
		else $objectArr = array($this->objectId);
	
		$path = "/Users/".USER_LOGIN."/Trash";
		$info = $this->objectFromPath(sanitize($path));
		
		//no collection, make one
		if (!$info) 
		{

			$path = "/Users/".USER_LOGIN;
			$info = $this->objectFromPath(sanitize($path));
			$parent = $info["id"];
			
			if (!$parent)
			{
				$this->throwError("There was an error creating the trash folder");
				return false;
			}

			//make the collection
			$opt = null;		
			$opt["name"] = "Trash";
			$opt["object_type"] = "collection";
			$opt["protected"] = 1;
			$opt["parent_id"] = $parent;
			$parentId = $this->create($opt);

			if (!$parentId)
			{
				$this->throwError("There was an error creating the trash folder");
				return false;
			}

			//make a bookmark for the user
			$opt = null;
			$opt["name"] = "Trash";
			$opt["account_id"] = USER_ID;
			$opt["protected"] = 1;
			$opt["object_id"] = $parentId;
			$id = $this->DB->insert("docmgr.dm_bookmark",$opt,"id");
			
		}
		else
		{

			$parentId = $info["id"];		
		
		}


		$oldParents = array();

		//now move everything into the trash folder
		foreach ($objectArr AS $object)
		{

			//init for later
			$LOCK = new DOCMGR_UTIL_LOCK($object);

		  $sql = "SELECT id,name,object_type,protected FROM docmgr.dm_object WHERE id='".$object."'";
		  $info = $this->DB->single($sql);
	
		  //if we can't find the obj, there's been an error.  Get out!!!	
		  if (!$info) {
	
		    $this->throwError("Could not find object \"".$info["name"]."\" for removal");
		    return false;
	
		  } else if ($LOCK->isLocked($info["id"])) {
	
		  	$this->throwError("\"".$info["name"]."\" is locked");
		  	return false;

		  } else if ($info["protected"]=="t") {
	
		  	$this->throwError("\"".$info["name"]."\" is a protected object and cannot be deleted");
		  	return false;
	
		  }

			//make sure we are allow to edit anything in the destination folder, since we are 
		  //moving things there
		  $cb = DOCMGR_UTIL_OBJPERM::getUser($object);
		  if (!DOCMGR_UTIL_OBJPERM::check($cb,"admin"))
		  {
		    $this->throwError("You do not have permissions to delete this ".$info["name"]);
				return false;
			}

			//get the old parents for later
			$sql = "SELECT parent_id FROM docmgr.dm_object_parent WHERE object_id IN (".implode(",",$objectArr).");";
			$info = $this->DB->fetch($sql,1);
			$oldParents = array_merge($oldParents,$info["parent_id"]);

			//delete all parents of this object
			$sql = "DELETE FROM docmgr.dm_object_parent WHERE object_id='$object'";
			$this->DB->query($sql);		

			//get the object's name
			$sql = "SELECT name FROM docmgr.dm_object WHERE id='$object'";
			$info = $this->DB->single($sql);		
			$fileName = $info["name"];

			//keep going until we find one that doesn't exist     
			while (!checkObjName($this->conn,$fileName,$parentId))
			{
				$fileName = $this->incrementName($fileName);
			}

			//update if it had to be changed
			if ($fileName != $info["name"])
			{

				$sql = "UPDATE docmgr.dm_object SET name='".addslashes($fileName)."' WHERE id='$object'";
				$this->DB->query($sql);			
			
			}

			//set it up in the trash folder
			$opt = null;
			$opt["parent_id"] = $parentId;
			$opt["object_id"] = $object;
			$this->DB->insert("docmgr.dm_object_parent",$opt);

		}

		//update trash folder size
		$this->updateCollectionSizes($parentId);		  
		$this->updateCollectionSizes($oldParents);

	}

	//empties the user's trash folder
	public function emptyTrash() 
	{
	
		$path = "/Users/".USER_LOGIN."/Trash";
		$info = $this->objectFromPath(sanitize($path));
		
		if (!$info) return false;
		
		$sql = "SELECT id FROM docmgr.dm_view_objects WHERE parent_id='".$info["id"]."'";
		$list = $this->DB->fetch($sql);
	
		for ($i=0;$i<$list["count"];$i++)
		{
			$o = new DOCMGR_OBJECT($list[$i]["id"]);
			$o->delete();
		}
	
	}

	//gets content only for a file or docmgr document
	public function getContent()
	{
	
		if ($this->objectInfo["object_type"]=="document")
		{

			$d = new DOCMGR_DOCUMENT($this->objectId);
			return $d->get();		
		
		}
		else
		{
		
			//make sure it passes back the content of the file in the response
			$d = new DOCMGR_FILE($this->objectId);
			return $d->get("contentonly");		
		
		}
	
	}

	/************************************************************
		FUNCTION:	getProperties
		PURPOSE:	(webdav only) pulls an object's properties as
							set by a webdav client
		INPUTS:		none
		RETURNS:	none
		OUTPUTS:	properties -> array
	************************************************************/
	public function getProperties()
	{

		if (!$this->objectId) $this->objectId = "0";
		
		$sql = "SELECT data FROM docmgr.dm_properties WHERE object_id='".$this->objectId."'";
		$info = $this->DB->single($sql);

		$this->PROTO->add("properties",unserialize($info["data"]));

		if ($this->DB->error()) $this->throwError($this->DB->error());
	
	}

	/************************************************************
		FUNCTION:	saveProperties
		PURPOSE:	(webdav only) save's an object's properties as
							set by a webdav client
		INPUTS:		apidata:
								data -> array of properties set by client
		RETURNS:	none
		OUTPUTS:	none;
	************************************************************/
	public function saveProperties()
	{

		if (!$this->objectId) $this->objectId = "0";
		
		$sql = "SELECT data FROM docmgr.dm_properties WHERE object_id='".$this->objectId."'";
		$info = $this->DB->single($sql);

		if (is_array($this->apidata["properties"])) $this->apidata["properties"] = serialize($this->apidata["properties"]);

		$opt = null;
		$opt["data"] = $this->apidata["properties"];

		if ($info)
		{
			$opt["where"] = "object_id='".$this->objectId."'";
			$this->DB->update("docmgr.dm_properties",$opt);
		}
		else
		{
			$opt["object_id"] = $this->objectId;
			$this->DB->insert("docmgr.dm_properties",$opt);
		}

	}

	protected function updateCollectionSizes($objId=null)
	{

		if (!$objId) $objId = $this->objectId;

		//convert to an array
		if (!is_array($objId)) $objId = array($objId);

		//loop through and set it all up		
		foreach ($objId AS $obj)
		{
	
			$paths = DOCMGR_UTIL_COMMON::objectIdAllPaths($obj);
			
			foreach ($paths AS $path)
			{
		
				//set the filesize of all collections to the sume of it's children
				$arr = explode(",",$path);
				
				//figure out which one are collections
				$sql = "SELECT id FROM docmgr.dm_object WHERE id in (".$path.") AND object_type='collection'";
				$list = $this->DB->fetch($sql,1);
				
				$arr = array_values(array_intersect($arr,$list["id"]));
				
				for ($i=0;$i<count($arr);$i++)
				{
				
					$sql = "UPDATE docmgr.dm_object SET filesize=
										(
											SELECT sum(filesize::int) FROM docmgr.dm_view_objects WHERE parent_id='".$arr[$i]."'
										)
										WHERE id='".$arr[$i]."'";
					$this->DB->query($sql);
				
				}	
					
			
			}
	
		}
		
	}
	
}
